"""
A map is composed of a series of tiles (individual images) with each tile
corresponding to a different type of texture (grass, dirt, etc.). For each
tile we have properties such as whether it's a barrier or a teleport tile.
We also specifying a starting tile for the board so we know where to render
everything.  This poses the question of how do we manage the collection of
tiles as a single object?

By using a class, we can treat the tiles as a collection with a simple
interface.  The class provides move(), move_up(), move_down(), move_left(),
and move_right() methods for manipulating the portion of the map shown on
screen.

The window's position is fixed with corners at (0, 0) and (window_width,
window_height).  To show different parts of the map, we actually move the
map itself.
"""

from pyglet.graphics import Batch
from pyglet.sprite import Sprite

from creatures.common.map_system import Grid2D
from creatures.common import geometry_utilities

class MapRenderer(object):
    def __init__(self, tileset, map_data, offset_coordinates, board_start=None):
        """
        The MapRenderer class is used to rendering a map on screen, manipulating
        the map (e.g, moving it), and providing information about about the map
        such as barriers and teleports.  Since the map is defined using a grid of
        tiles, the MapRenderer maps those tiles to the window's x-y coordinate
        system.  Thus, the MapRenderer can be used to determine whether sprites
        are on top of teleports or barriers.

        tileset

        map_data

        board_start -- Override the board_start tile defined in the map data. Useful for
        teleporting to different areas on the same map.

        offset_coordinates -- Sets the offset coordinates. The map is placed by aligning
        the lower-left corner of a tile to the offset coordinates.  The particular tile
        depends on board_start.

        board_start -- Sets behavior and possibly tile used when aligning the starting
        coordinates of the map. By default, board_start is None and the lower-left corner of
        the tile defined in board_start is aligned to the offset coordinates.  If board_start
        is False, the map is aligned so that tile 0, 0 aligns with the offset coordinates.
        If a tuple of (tile_col, tile_row) is provided for board_start, then the map is aligned
        such that the lower-left corner of the given tile aligns with the offset coordinates.

        Note: It is assumed that the 0, 0 tile is the lower left hand tile.
        """
        self.map_data = map_data
        self.tileset = tileset

        # tile width and heights -- used to compute tile positions, map
        # between tile grid and X, Y coordinates
        self.tile_width = self.tileset.tile_width
        self.tile_height = self.tileset.tile_height

        # a batch is used to speed up rendering of multiple sprites
        self.batch = Batch()

        self.rows = self.map_data.rows
        self.columns = self.map_data.columns

        # The map is stored as a 2D grid of sprites. Sprites are images
        # with locations on screen -- they allow us to easily move and re-use
        # images without having to load separate copies of the images in memory.
        # 2D array that holds data about each tile
        # grid is organized such that 0, 0 refers to the upper-left most tile
        # (e.g, matrix indexing). This contrasts with how things are drawn where
        # 0,0 refers to the bottom left.
        self.grid = Grid2D(rows = self.rows, cols = self.columns)

        map_height = self.rows * self.tile_height

        # initialize sprites and positions
        for (c, r), tile_struct in self.map_data.grid:
            x = c * self.tile_width
            y = r * self.tile_height
            tile = self.tileset.tiles[tile_struct.tile_id]
            self.grid[c, r] = Sprite(tile.image, x, y, batch=self.batch)

        # Current location of the lower left corner of the map
        self.x_offset = 0
        self.y_offset = 0

        # constant for number of pixels to move
        self.ROW_SPEED = self.tile_height / 4
        self.COL_SPEED = self.tile_width / 4

        # move board to starting position
        if len(self.grid) != 0:
            start_x, start_y = offset_coordinates
            if board_start is not False:
                if board_start is not None:
                    start_tile = self.grid[board_start]
                else:
                    start_tile = self.grid[self.map_data.board_start]
                start_x -= start_tile.x
                start_y -= start_tile.y
            self.move(start_x, start_y)

    def outside_map(self, pos_x, pos_y):
        """
        Return True if coordinate defined by pos_x and pos_y are outside
        the map, otherwise return False.
        """
        width = self.columns * self.tile_width
        height = self.rows * self.tile_height
        # check 1-pixel border directly outside map
        return not \
            geometry_utilities.point_in_box(pos_x, pos_y, self.x_offset - 1, \
                                                   self.y_offset - 1, width + 2, \
                                                   height + 2)

    def round_to_tile_corner(self, x, y):
        # round x, y to the x, y position to tile's
        # lower left corner
        tile_col, tile_row = self.coordinate_to_tile(x, y)
        tile_x = tile_col * self.tile_width  + self.x_offset
        tile_y = tile_row * self.tile_height + self.y_offset
        return tile_x, tile_y

    def coordinate_to_tile(self, x, y):
        # map x, y coordinates to containing tile's col and row
        tile_col = (x - self.x_offset) // self.tile_width
        tile_row = (y - self.y_offset) // self.tile_height
        return tile_col, tile_row

    def move(self, x_offset, y_offset):
        """
        Move lower left hand corner of map to x_offset, y_offset. Coordinates
        specified in coordinate system of window -- 0, 0 means lower left hand
        corner of screen.
        """
        self.x_offset = x_offset
        self.y_offset = y_offset
        for (c, r), tile in self.grid:
            tile.x = x_offset + c * self.tile_width
            tile.y = y_offset + r * self.tile_height

    def move_down(self):
        """
        Move map down by one increment.  Increment defined internally.
        """
        self.move(self.x_offset, self.y_offset + self.COL_SPEED)

    def move_right(self):
        """
        Move map right by one increment.  Increment defined internally.
        """
        self.move(self.x_offset - self.COL_SPEED, self.y_offset)

    def move_left(self):
        """
        Move map left by one increment.  Increment defined internally.
        """
        self.move(self.x_offset + self.ROW_SPEED, self.y_offset)

    def move_up(self):
        """
        Move map up by one increment.  Increment defined internally.
        """
        self.move(self.x_offset, self.y_offset - self.ROW_SPEED)

    def draw(self):
        """
        Draw map.  Should be called after everything else, otherwise
        it will draw over other objects.
        """
        self.batch.draw()

    def delete_sprites(self):
        for (c, r), sprite in self.grid:
            sprite.delete()

class MapRendererFactory(object):
    """
    Allows hiding of tileset_factory from classes that don't need
    to know about it.
    """
    def __init__(self, tileset_factory):
        self.tileset = tileset_factory.load_tileset()

    def create_renderer(self, map_data, offset_coordinates, board_start=None):
        return MapRenderer(self.tileset, map_data, offset_coordinates, \
                               board_start=board_start)
