from collections import deque
import os

from creatures.common import constants

"""
This module contains classes for modeling maps, loading and saving maps,
and various utilities.  Classes include:

* Grid2D -- a simple implementation of a 2D-indexable array
* MapData -- class for holding map data such as tile types and tile
attributes
* TileStruct -- class used for holding per-tile attributes
* MapFactory -- used for loading and saving maps from / to files

A map is composed of a series of tiles (individual images) with each tile
corresponding to a different type of texture (grass, dirt, etc.). For each
tile we have properties such as whether it's a barrier or a teleport tile.
We also specifying a starting tile for the board so we know where to render
everything.
"""

class Grid2D(object):
    """
    Simple implementation of a 2D array for storing tiles in map.
    """
    def __init__(self, data=None, cols=0, rows=0):
        if data is not None:
            self.rows = data[0]
            self.cols = data[0][0]
            self.data = [None] * (self.rows * self.cols)
            for r, row in enumerate(data):
                for c, element in enumerate(row):
                    self[c, r] = element
        else:
            self.rows = rows
            self.cols = cols
            self.data = [None] * (self.rows * self.cols)

    def __getitem__(self, key):
        c, r = key
        return self.data[self.rows * c + r]

    def __setitem__(self, key, value):
        c, r = key
        self.data[self.rows * c + r] = value

    def __len__(self):
        return self.cols * self.rows

    def __iter__(self):
        for r in xrange(self.rows):
            for c in xrange(self.cols):
                yield (c, r), self[c, r]

class TileStruct(object):
    """
    Contains information needed for each tile of the map.
    """
    def __init__(self, tile_id, barrier=False, teleport=False, teleport_destination = None, teleport_land = None):
        self.tile_id = tile_id
        self.barrier = barrier
        self.teleport = teleport
        self.teleport_destination = teleport_destination
        # Tuple of (col, row).  0, 0 is lower left corner of board
        self.teleport_land = teleport_land

class MapData(object):
    """
    Contains all map data.  Does not maintain any sort of state data.
    """
    def __init__(self, map_tile_ids=tuple(), rows=0, cols=0, barriers=tuple(), \
                     board_start=(0, 0), teleport_tiles=tuple(), grid=None):
        """
        map_tile_ids -- Sequence of tile ids

        rows

        cols

        barriers -- Sequence of (col, row) tuples for barriers

        board_start -- Tuple of (col, row) indicating tile that will be aligned with
        character on initial load of map

        teleport_tiles -- Tuples of (destination_map_name, (col, row), (target_col, target_row))
        indicating teleport tiles.  Col and row correspond to location on current map
        while target_col, target_row correspond to location on destination map.

        grid -- Used to initialize from other MapData instances.

        Note that all indices assume 0, 0 is the lower left hand tile.
        """
        # Number of rows
        self.rows = rows
        # Number of columns
        self.columns = cols
        # Tuple of (col, row) indicating tile that will be aligned with
        # character on initial load of map
        self.board_start = board_start

        # Stores all per-tile information such as tileset id, barrier, teleport data
        self.grid = grid
        if grid == None:
            self.grid = Grid2D(cols = self.columns, rows = self.rows)

        for i, tile_id in enumerate(map_tile_ids):
            c = i % self.columns
            r = self.rows - (i // self.columns) - 1
            self.grid[c, r] = TileStruct(tile_id)

        for c, r in barriers:
            self.grid[c, r].barrier = True

        for destination, (c0, r0), (c1, r1) in teleport_tiles:
            self.grid[c0, r0].teleport = True
            self.grid[c0, r0].teleport_destination = destination
            self.grid[c0, r0].teleport_land = (c1, r1)

class MapFactory(object):
    """
    Used to load data from map files and generate Map objects.
    """
    def __init__(self, map_dir):
        """
        map_dir -- path to directory containing maps
        """
        self.map_dir = map_dir

    def load_first_map(self):
        """
        Return first map
        """
        return self.load_map(constants.FIRST_MAP)

    def create_blank_map(self):
        return MapData()

    def load_map(self, map_name):
        """
        Load map with given name.  Name is not a full path.
        """
        fl = open(self.map_dir + os.sep + map_name + constants.MAP_EXTENSION)
        s = fl.read()
        fl.close()
        tokens = deque(s.split())
        # discard columns keyword
        _ = tokens.popleft()
        cols = int(tokens.popleft())
        # discard rows keyword
        _ = tokens.popleft()
        rows = int(tokens.popleft())
        # discard grid keyword
        _ = tokens.popleft()
        map_tile_ids = []
        for i in xrange(rows * cols):
            tile_id = int(tokens.popleft())
            map_tile_ids.append(tile_id)
        # discard barriers keyword
        _ = tokens.popleft()
        num_barriers = int(tokens.popleft())
        barriers = []
        for i in xrange(num_barriers):
            c = int(tokens.popleft())
            r = int(tokens.popleft())
            barriers.append((c, r))
        # discard boardstart keyword
        _ = tokens.popleft()
        c = int(tokens.popleft())
        r = int(tokens.popleft())
        board_start = (c, r)

        # discard teleports keyword
        _ = tokens.popleft()
        num_teleports = int(tokens.popleft())
        teleport_tiles = []
        for i in xrange(num_teleports):
            destination = tokens.popleft()
            c0 = int(tokens.popleft())
            r0 = int(tokens.popleft())
            c1 = int(tokens.popleft())
            r1 = int(tokens.popleft())
            teleport_launch_tile = (c0, r0)
            teleport_land_tile = (c1, r1)
            teleport_tiles.append((destination, teleport_launch_tile, teleport_land_tile))
        return MapData(map_tile_ids, rows, cols, barriers, board_start, teleport_tiles)


    def save_map(self, map, map_name):
        """
        Save map with given name.  Name is not full path.
        """
        fl = open(self.map_dir + os.sep + map_name + constants.MAP_EXTENSION, "w")
        fl.write("Columns %s\n" % map.grid.cols)
        fl.write("Rows %s\n" % map.grid.rows)
        tile_ids = []
        barriers = []
        teleports = []
        fl.write("Grid ")
        for i in xrange(map.grid.rows):
            r = map.grid.rows - i - 1
            for c in xrange(map.grid.cols):
                tile = map.grid[c, r]
                tile_ids.append(tile.tile_id)
                if tile.barrier:
                    barriers.append((c, r))
                if tile.teleport:
                    teleports.append((tile.teleport_destination, (c, r), tile.teleport_land))
                fl.write("%s " % tile.tile_id)
            fl.write("\n")
        fl.write("Barriers %s " % len(barriers))
        for barrier in barriers:
            fl.write("%s %s " % barrier)
        fl.write("\n")
        fl.write("BoardStart %s %s\n" % map.board_start)
        fl.write("Teleports %s " % len(teleports))
        for teleport_map_name, (c, r), (land_c, land_r) in teleports:
            fl.write("%s %s %s %s %s " % (teleport_map_name, c, r, land_c, land_r))
        fl.write("\n")
        fl.close()
