"""
Extension of the MapRenderer in common.  This adds support for highlighting tiles
based on attributes.
"""

from pyglet.graphics import Batch
from pyglet.sprite import Sprite

from creatures.editor import drawing_utilities

from creatures.common.map_renderer import MapRenderer
from creatures.common.map_system import Grid2D

class EditorMapRenderer(MapRenderer):
    def __init__(self, tileset, map_data, offset_coordinates, board_start=None):
        """
        tileset

        map_data

        board_start -- Override the board_start tile defined in the map data. Useful for
        teleporting to different areas on the same map.

        offset_coordinates -- Sets the offset coordinates. The map is placed by aligning
        the lower-left corner of a tile to the offset coordinates.  The particular tile
        depends on board_start.

        board_start -- Sets behavior and possibly tile used when aligning the starting
        coordinates of the map. By default, board_start is None and the lower-left corner of
        the tile defined in board_start is aligned to the offset coordinates.  If board_start
        is False, the map is aligned so that tile 0, 0 aligns with the offset coordinates.
        If a tuple of (tile_col, tile_row) is provided for board_start, then the map is aligned
        such that the lower-left corner of the given tile aligns with the offset coordinates.

        Note: It is assumed that the 0, 0 tile is the lower left hand tile.
        """
        super(EditorMapRenderer, self).__init__(tileset, map_data, offset_coordinates, \
                                                    board_start=board_start)

        # Coordinates of tile over which the mouse is currently
        # hovering. Using coordinates instead of col, row since
        # tile map not be part of map
        self.current_tile_x = None
        self.current_tile_y = None

        # Speed at which the board is moved in pixels/update
        self.ROW_SPEED = self.tile_width // 2
        self.COL_SPEED = self.tile_height // 2

        # Tile user has chosen (used by the TileChooserInterface)
        self.chosen_tile = None

    def set_current_tile(self, x, y):
        """
        Sets the tile that the mouse located at x, y is currently
        hovering over.
        """
        self.current_tile_x, self.current_tile_y = self.round_to_tile_corner(x, y)

    def set_chosen_tile(self, x, y):
        """
        Set the tile that the user has chosen via the current mouse coordinates.
        Used in the Tile Chooser dialog.  This method checks if the chosen tile is
        valid.  It returns True if valid, otherwise, it returns False.
        """
        if self.outside_map(x, y):
            return False
        coord = self.coordinate_to_tile(x, y)
        if self.map_data.grid[coord].barrier:
            return False
        if self.map_data.grid[coord].teleport:
            return False
        self.chosen_tile = coord
        return True

    def get_chosen_tile(self):
        """
        Return column and row of currently chosen tile.
        """
        return self.chosen_tile

    def draw_tile_attribute_highlights(self):
        """
        Draw colored borders around tiles with different properties
        (board_start, barrier, teleport, currently chosen tile, etc.)
        """
        for coordinates, tile in self.map_data.grid:
            color = None
            if coordinates == self.map_data.board_start:
                color = drawing_utilities.ORANGE
            elif tile.barrier:
                color = drawing_utilities.RED
            elif tile.teleport:
                color = drawing_utilities.BLUE
            elif coordinates == self.chosen_tile:
                color = drawing_utilities.YELLOW
            else:
                continue
            sprite = self.grid[coordinates]
            drawing_utilities.draw_rectangle(color, sprite.x, sprite.y,
                                             self.tile_width,
                                             self.tile_height)
        if self.current_tile_x != None:
            drawing_utilities.draw_rectangle(drawing_utilities.PURPLE,
                                             self.current_tile_x, self.current_tile_y,
                                             self.tile_width, self.tile_height)


    def draw(self):
        """
        Draw map.  Should be called after everything else, otherwise
        it will draw over other objects.
        """
        super(EditorMapRenderer, self).draw()
        self.draw_tile_attribute_highlights()

class EditorMapRendererFactory(object):
    """
    Allows hiding of tileset_factory from classes that don't need
    to know about it.
    """
    def __init__(self, tileset_factory):
        self.tileset = tileset_factory.load_tileset()
        self.tile_width = self.tileset.tile_width
        self.tile_height = self.tileset.tile_height

    def create_renderer(self, map_data, offset_coordinates, board_start=None):
        return EditorMapRenderer(self.tileset, map_data, offset_coordinates, \
                               board_start=board_start)

    def create_shifted_renderer(self, map_data, old_map_renderer, col_shift,
                                row_shift):
        # shift EditorMapRenderer offsets so that board is replaced "transparently"
        x_shift = col_shift * self.tile_width
        y_shift = row_shift * self.tile_height
        x_offset = old_map_renderer.x_offset - x_shift
        y_offset = old_map_renderer.y_offset - y_shift
        offsets = x_offset, y_offset

        return EditorMapRenderer(self.tileset, map_data, offsets, board_start=False)
