"""
High-level game module.  The World class keeps track of the map and characters,
handles user input, and facilitates map mechanics.
"""

import sys

from pyglet.window import key

from creatures.game.character_system import CharacterFactory
from creatures.game.collision_detection import CollisionDetector
from creatures.common import constants
from creatures.common.map_system import MapFactory


class World(object):
    def __init__(self, key_handler, window_height, window_width, map_renderer_factory,
                 map_name=None):
        """
        The World class keeps track of the map and characters,
        handles user input, and facilitates map mechanics.

        map_name -- String.  If given, loads given map.  If not, loads default
        first map.
        """
        self.key_handler = key_handler
        self.window_height = window_height
        self.window_width = window_width
        self.map_renderer_factory = map_renderer_factory

        # start in center of screen
        window_center = self.window_width / 2, self.window_height / 2

        self.character_factory = CharacterFactory(constants.IMAGE_DIR)
        self.character = self.character_factory.character1(window_center)
        self.character_start = self.character.x, self.character.y

        self.map_factory = MapFactory(constants.MAP_DIR)
        if map_name == None:
          self.map = self.map_factory.load_first_map()
        else:
          self.map = self.map_factory.load_map(map_name)
        self.map_renderer = self.map_renderer_factory.create_renderer(self.map,
                                                                      self.character_start)
        self.collision_detector = CollisionDetector(self.map, self.map_renderer)

    def __call__(self, dt):
        """
        This will be our global "update" function. All game
        mechanics are handled here.
        """

        # quit program
        if self.key_handler[key.Q]:
            sys.exit()

        # reset to default orientation
        self.character.set_orientation_straight()

        # character is fixed in center of screen
        # move map and other objects instead
        if self.key_handler[key.UP]:
            self.map_renderer.move_up()
            self.character.set_orientation_up()
            if self.collision_detector.collision(self.character):
                self.map_renderer.move_down()

        if self.key_handler[key.DOWN]:
            self.map_renderer.move_down()
            self.character.set_orientation_down()
            if self.collision_detector.collision(self.character):
                self.map_renderer.move_up()

        if self.key_handler[key.LEFT]:
            self.map_renderer.move_left()
            self.character.set_orientation_left()
            if self.collision_detector.collision(self.character):
                self.map_renderer.move_right()

        if self.key_handler[key.RIGHT]:
            self.map_renderer.move_right()
            self.character.set_orientation_right()
            if self.collision_detector.collision(self.character):
                self.map_renderer.move_left()


        # handle teleporting between locations on the same or different maps
        teleport_bool, teleport_map, teleport_destination = self.collision_detector.teleport(self.character)
        if teleport_bool:
            self.map = self.map_factory.load_map(teleport_map)
            self.map_renderer.delete_sprites()
            self.map_renderer = \
                self.map_renderer_factory.create_renderer(self.map,
                                                          self.character_start,
                                                          board_start=teleport_destination)
            self.collision_detector = CollisionDetector(self.map, self.map_renderer)



    def draw(self):
        # draw map first as it's below everything first
        self.map_renderer.draw()
        self.character.draw()
