import math
import random
import sys

import pyglet
from pyglet.gl import *
from pyglet.window import key
from pyglet.window import mouse

from creatures.common import constants
from creatures.common.tileset import TilesetFactory
from creatures.editor.editor_user_interface import EditorUserInterface
from creatures.editor.tile_type_chooser_interface import TileTypeChooserInterface
from creatures.editor.map_editor_interface import MapEditorInterface
from creatures.editor.map_renderer import EditorMapRendererFactory

WINDOW_WIDTH = 1280
WINDOW_HEIGHT = 800

# create the window, but keep it offscreen until we are done with setup
window = pyglet.window.Window(WINDOW_WIDTH, WINDOW_HEIGHT, visible=False, caption="Game Editor")

# centre the window on whichever screen it is currently on (in case of multiple monitors)
window.set_location(window.screen.width/2 - window.width/2, window.screen.height/2 - window.height/2)

# handle the window resize event
@window.event
def on_resize(width, height):
    glViewport(0, 0, width, height)

    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    # setup a fixed dimension orthographic projection
    # this avoids gameplay problems if the window resolution is changed
    glOrtho(0, WINDOW_WIDTH, 0, WINDOW_HEIGHT, -1, 1)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

    # tell pyglet that we have handled the event, prevents the default handler from running
    return pyglet.event.EVENT_HANDLED

# setup a key handler to track keyboard input
keymap = pyglet.window.key.KeyStateHandler()
window.push_handlers(keymap)

# clear the window and draw the scene
@window.event
def on_draw():
    window.clear()
    user_interface.draw()

@window.event
def on_mouse_motion(x, y, dx, dy):
    user_interface.set_mouse_position(x, y)

@window.event
def on_mouse_release(x, y, button, modifiers):
    if button == mouse.LEFT:
        user_interface.set_mouse_clicked()


# clear and flip the window, otherwise we see junk in the buffer before the first frame
window.clear()
window.flip()

# make the window visible at last
window.set_visible(True)

# update callback
key_handler = key.KeyStateHandler()
window.push_handlers(key_handler)

map_name = None
if len(sys.argv) == 2:
    map_name = sys.argv[1]

# load tile sets and create game objects
tileset_factory = TilesetFactory(constants.IMAGE_DIR, constants.TILESETS)
map_renderer_factory = EditorMapRendererFactory(tileset_factory)
map_editor_interface = MapEditorInterface(key_handler, WINDOW_HEIGHT, WINDOW_WIDTH, \
                                             map_renderer_factory, map_name=map_name)
tile_type_chooser_interface = TileTypeChooserInterface(tileset_factory, WINDOW_HEIGHT, WINDOW_WIDTH)
user_interface = EditorUserInterface(key_handler, WINDOW_HEIGHT, WINDOW_WIDTH, \
                                         map_editor_interface, tile_type_chooser_interface)
user_interface.draw()

# schedule the update function, 30 times per second
pyglet.clock.schedule_interval(user_interface, 1.0/30.0)


# finally, run the application
pyglet.app.run()
